/* may only be used in accordance with the terms of the GNU General Public License */

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>

#ifdef USE_GTK

#include <string.h>
#include <gtk/gtk.h>
#include "gharmonic.h"

#endif

#include "charmonic.h"

/* main() either just runs c_do() - the CLI/server main loop, or 
 * if compiled with GTK support and invoked as gharmonic, 
 * runs g_do() - the GUI/client loop as well as the CLI/server as a 
 * child process and talks to it via a pipe
 */

int main(int argc, char *argv[])
{
  int result = 0;
  int fd;
#ifdef USE_GTK
  int pfd[2];

  /* no "gharmonic" in name - just run the CLI/server */
  if (strstr(argv[0], "gharmonic") == NULL) {
#endif				/* if no gtk, then life is simple - just run the CLI/server */
    if (argc > 1) {
      if (argv[1][0] != '-') {
	fd = open(argv[1], O_RDONLY);
	if (fd != (-1)) {
	  if (dup2(fd, STDIN_FILENO) == (-1)) {
	    perror("Can not dup2 input file descriptor");
	    exit(1);
	  }
	} else {
	  perror("Can not open file");
	  exit(1);
	}
      }
    }
    result = c_do();
#ifdef USE_GTK
  }
  /* GTK installed and gharmonic in name - run the CLI/server and GUI/client */
  else {
    if (pipe(pfd)) {
      perror("Can't create pipe");
      result = 1;
    } else {
      fflush(stdout);
      switch (fork()) {
      case -1:
	/* parent - failure */
	perror("Can't fork");
	result = 1;
	break;
      case 0:
	/* child - run CLI/server */
	close(pfd[1]);
	if (dup2(pfd[0], STDIN_FILENO) == (-1)) {
	  perror("Can't dup2");
	  result = 1;
	} else {
	  result = c_do();
	}
	break;
      default:
	/* parent - run GUI/client */
	close(pfd[0]);
	if (dup2(pfd[1], STDOUT_FILENO) == (-1)) {
	  perror("Can't dup2");
	  result = 1;
	} else {
	  gtk_init(&argc, &argv);
	  g_do();
	}
	break;
      }
    }
  }
#endif

  return result;
}

/* 21J.gf83Iap.Q */
